/***

The MIT License

Copyright (c) 2018 Teppei Tamra (TAM)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

***/

#include "saoriview.h"
#include "ui_saoriview.h"
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QLabel>
#include <QScrollBar>
#include <QDebug>
#include <QDateTime>
#include <saoriapplication.h>

QList<SaoriView *> SaoriView::m_viewList;

SaoriView::SaoriView(const QString view, const QString account, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::SaoriView)
{
    ui->setupUi(this);
    m_viewname = view;
    m_account = account;
    m_viewList.append(this);

    auto saoriaccount = SaoriApplication::saori()->findAccount(m_account);
    if (saoriaccount) {
        connect(saoriaccount,&SaoriAccount::apiData,this,&SaoriView::recived);
        saoriaccount->getTimelineData(view);
    }
}

SaoriView::~SaoriView()
{
    delete ui;
    m_viewList.removeOne(this);
}

SaoriView *SaoriView::findView(const QString view, const QString account)
{
    for(auto v:m_viewList) {
        if ((v->m_viewname == view) && (v->m_account == account)) return v;
    }
    return nullptr;
}

const QString SaoriView::statusParser(const QJsonObject json)
{
    QString result;
    if (json.isEmpty()) return QString();
    result += "<div class=\"status\">";
    result += accountParser(json["account"].toObject());
    result += "<div class=\"created_at\">";
    QDateTime dt = QDateTime::fromString(json["created_at"].toString(),"yyyy-MM-ddTHH:mm:ss.zzzZ");
    dt.setTimeSpec(Qt::UTC);
    result += dt.toLocalTime().toString();
    result += "</div>";
    result += "<div class=\"content\">";
    result += json["content"].toString();
    result += "</div>";
    result += "</div>";
    return result;
}

const QString SaoriView::accountParser(const QJsonObject json)
{
    QString result;
    if (json.isEmpty()) return QString();
    result += "<div class=\"account\">";
    result += "<span class=\"avatar\"><hr />";
    result += "<img(";
    result += json["avatar"].toString();
    result += ") width=64 heght=64 style=\"float:left;\" /></span>";
    result += "<span class=\"display_name\">";
    result += json["display_name"].toString();
    result += "</span><span class=\"acct\">";
    result += json["acct"].toString();
    result += "</span>";
    result += "<div class=\"user_info\">";
    result += tr(" following: ");
    result += QString::number(json["following_count"].toInt());
    result += tr(" followers: ");
    result += QString::number(json["followers_count"].toInt());
    result += "</div>";
    result += "</div>";
    return result;
}

void SaoriView::recived(const QString timeline,const QByteArray data)
{
    if (timeline != m_viewname) return;
    QJsonArray json = QJsonDocument::fromJson(data).array();
    for (auto j:json) {
        auto entry = new SaoriViewEntry(ui->scrollAreaWidgetContents);
        entry->setObjectName(j.toObject()["id"].toString());
        entry->setContent(statusParser(j.toObject()));
        ui->scrollAreaWidgetContents->layout()->addWidget(entry);
    }
}
